@@ -656,7 +656,14 @@ static enum dhcp_token read_number (c, cfile)
 	cfile -> tlen = i;
 	cfile -> tval = cfile -> tokbuf;
 
-	return token;
+	/*
+	 * If this entire token from start to finish was "-", such as
+	 * the middle parameter in "42 - 7", return just the MINUS token.
+	 */
+	if ((i == 1) && (cfile->tokbuf[i] == '-'))
+		return MINUS;
+	else
+		return token;
 }
 
 static enum dhcp_token read_num_or_name (c, cfile)
@@ -918,6 +925,8 @@ intern(char *atom, enum dhcp_token dfv) {
 			return FIXED_ADDR;
 		if (!strcasecmp (atom + 1, "ixed-address6"))
 			return FIXED_ADDR6;
+		if (!strcasecmp (atom + 1, "ixed-prefix6"))
+			return FIXED_PREFIX6;
 		if (!strcasecmp (atom + 1, "ddi"))
 			return TOKEN_FDDI;
 		if (!strcasecmp (atom + 1, "ormerr"))
@@ -958,8 +967,14 @@ intern(char *atom, enum dhcp_token dfv) {
 	      case 'i':
 	      	if (!strcasecmp(atom+1, "a-na")) 
 			return IA_NA;
+	      	if (!strcasecmp(atom+1, "a-ta")) 
+			return IA_TA;
+	      	if (!strcasecmp(atom+1, "a-pd")) 
+			return IA_PD;
 	      	if (!strcasecmp(atom+1, "aaddr")) 
 			return IAADDR;
+	      	if (!strcasecmp(atom+1, "aprefix")) 
+			return IAPREFIX;
 		if (!strcasecmp (atom + 1, "nclude"))
 			return INCLUDE;
 		if (!strcasecmp (atom + 1, "nteger"))
@@ -1139,6 +1154,8 @@ intern(char *atom, enum dhcp_token dfv) {
 			return PACKET;
 		if (!strcasecmp (atom + 1, "ool"))
 			return POOL;
+		if (!strcasecmp (atom + 1, "refix6"))
+			return PREFIX6;
 		if (!strcasecmp (atom + 1, "seudo"))
 			return PSEUDO;
 		if (!strcasecmp (atom + 1, "eer"))
@@ -1359,6 +1376,8 @@ intern(char *atom, enum dhcp_token dfv) {
 			return TSFP;
 		if (!strcasecmp (atom + 1, "ransmission"))
 			return TRANSMISSION;
+		if (!strcasecmp(atom + 1, "emporary"))
+			return TEMPORARY;
 		break;
 	      case 'u':
 		if (!strcasecmp (atom + 1, "case"))
@@ -1389,7 +1408,7 @@ intern(char *atom, enum dhcp_token dfv) {
 			break;
 		}
 		if (!strcasecmp (atom + 1, "nauthenticated"))
-			return AUTHENTICATED;
+			return UNAUTHENTICATED;
 		if (!strcasecmp (atom + 1, "pdated-dns-rr"))
 			return UPDATED_DNS_RR;
 		if (!strcasecmp (atom + 1, "pdate"))
@@ -1416,6 +1435,8 @@ intern(char *atom, enum dhcp_token dfv) {
 			return NS_YXRRSET;
 		break;
 	      case 'z':
+		if (!strcasecmp (atom + 1, "erolen"))
+			return ZEROLEN;
 		if (!strcasecmp (atom + 1, "one"))
 			return ZONE;
 		break;
