@@ -52,7 +52,7 @@
  */
 
 /*
- * Portions Copyright (c) 2004,2008-2009 by Internet Systems Consortium, Inc. ("ISC")
+ * Portions Copyright (c) 2004,2008 by Internet Systems Consortium, Inc. ("ISC")
  * Portions Copyright (c) 1996-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -71,12 +71,12 @@
  *   950 Charter Street
  *   Redwood City, CA 94063
  *   <info@isc.org>
- *   https://www.isc.org/
+ *   http://www.isc.org/
  */
 
 #if defined(LIBC_SCCS) && !defined(lint)
 static const char sccsid[] = "@(#)res_send.c	8.1 (Berkeley) 6/4/93";
-static const char rcsid[] = "$Id: res_send.c,v 1.8.580.5 2009/08/06 22:05:34 sar Exp $";
+static const char rcsid[] = "$Id: res_send.c,v 1.11 2008/01/23 22:49:54 marka Exp $";
 #endif /* LIBC_SCCS and not lint */
 
 /*
