@@ -3,7 +3,7 @@
    Subroutines that support minires tracing... */
 
 /*
- * Copyright (c) 2004 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004,2007 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 2001-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -22,19 +22,23 @@
  *   950 Charter Street
  *   Redwood City, CA 94063
  *   <info@isc.org>
- *   https://www.isc.org/
+ *   http://www.isc.org/
  *
  * This software has been written for Internet Systems Consortium
  * by Ted Lemon, as part of a project for Nominum, Inc.   To learn more
- * about Internet Systems Consortium, see https://www.isc.org/.  To
+ * about Internet Systems Consortium, see http://www.isc.org/.  To
  * learn more about Nominum, Inc., see ``http://www.nominum.com''.
  */
 
+#include "dhcpd.h"
 #include <omapip/omapip_p.h>
 
 #include "minires/minires.h"
 #include "arpa/nameser.h"
 
+#include <errno.h>
+
+#if defined(TRACING)
 static void trace_mr_output_input (trace_type_t *, unsigned, char *);
 static void trace_mr_output_stop (trace_type_t *);
 static void trace_mr_input_input (trace_type_t *, unsigned, char *);
@@ -43,6 +47,7 @@ static void trace_mr_statp_input (trace_type_t *, unsigned, char *);
 static void trace_mr_statp_stop (trace_type_t *);
 static void trace_mr_randomid_input (trace_type_t *, unsigned, char *);
 static void trace_mr_randomid_stop (trace_type_t *);
+#endif /* TRACING */
 trace_type_t *trace_mr_output;
 trace_type_t *trace_mr_input;
 trace_type_t *trace_mr_statp;
@@ -61,8 +66,6 @@ time_t trace_mr_time (time_t *);
 int trace_mr_select (int, fd_set *, fd_set *, fd_set *, struct timeval *);
 unsigned int trace_mr_res_randomid (unsigned int);
 
-extern TIME cur_time;
-
 #if defined (TRACING)
 void trace_mr_init ()
 {
@@ -85,7 +88,6 @@ void trace_mr_statp_setup (res_state statp)
 	unsigned buflen = 0;
 	char *buf = (char *)0;
 	isc_result_t status;
-	u_int32_t id;
 	int i;
 
 	if (trace_playback ()) {
@@ -404,15 +406,14 @@ int trace_mr_select (int s, fd_set *r, fd_set *w, fd_set *x, struct timeval *t)
 
 unsigned int trace_mr_res_randomid (unsigned int oldid)
 {
-	u_int32_t id;
 	int rid = oldid;
 #if defined (TRACING)
+	u_int32_t id;
 	unsigned buflen = 0;
 	char *buf = (char *)0;
 	isc_result_t status;
 
 	if (trace_playback ()) {
-		int nscount;
 		status = trace_get_packet (&trace_mr_randomid, &buflen, &buf);
 		if (status != ISC_R_SUCCESS) {
 			log_error ("trace_mr_statp: no statp packet found.");
