@@ -3,7 +3,7 @@
    DHCP Server Daemon. */
 
 /*
- * Copyright (c) 2004-2010 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2009 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1996-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -22,22 +22,21 @@
  *   950 Charter Street
  *   Redwood City, CA 94063
  *   <info@isc.org>
- *   https://www.isc.org/
+ *   http://www.isc.org/
  *
  * This software has been written for Internet Systems Consortium
  * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
  * To learn more about Internet Systems Consortium, see
- * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
+ * ``http://www.isc.org/''.  To learn more about Vixie Enterprises,
  * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
  * ``http://www.nominum.com''.
  */
 
-static const char copyright[] =
-"Copyright 2004-2010 Internet Systems Consortium.";
-static const char arr [] = "All rights reserved.";
-static const char message [] = "Internet Systems Consortium DHCP Server";
-static const char url [] =
-"For info, please visit https://www.isc.org/software/dhcp/";
+static char copyright[] =
+"Copyright 2004-2009 Internet Systems Consortium.";
+static char arr [] = "All rights reserved.";
+static char message [] = "Internet Systems Consortium DHCP Server";
+static char url [] = "For info, please visit http://www.isc.org/sw/dhcp/";
 
 #include "dhcpd.h"
 #include <omapip/omapip_p.h>
@@ -45,8 +44,19 @@ static const char url [] =
 #include <errno.h>
 #include <limits.h>
 #include <sys/types.h>
+#include <sys/time.h>
 #include <signal.h>
 
+#if defined (PARANOIA)
+#  include <sys/types.h>
+#  include <unistd.h>
+#  include <pwd.h>
+/* get around the ISC declaration of group */
+#  define group real_group 
+#    include <grp.h>
+#  undef group
+#endif /* PARANOIA */
+
 static void usage(void);
 
 struct iaddr server_identifier;
@@ -174,6 +184,7 @@ static void omapi_listener_start (void *foo)
 {
 	omapi_object_t *listener;
 	isc_result_t result;
+	struct timeval tv;
 
 	listener = (omapi_object_t *)0;
 	result = omapi_generic_new (&listener, MDL);
@@ -188,11 +199,29 @@ static void omapi_listener_start (void *foo)
 	if (result != ISC_R_SUCCESS) {
 		log_error ("Can't start OMAPI protocol: %s",
 			   isc_result_totext (result));
-		add_timeout (cur_time + 5, omapi_listener_start, 0, 0, 0);
+		tv.tv_sec = cur_time + 5;
+		tv.tv_usec = 0;
+		add_timeout (&tv, omapi_listener_start, 0, 0, 0);
 	}
 	omapi_object_dereference (&listener, MDL);
 }
 
+#if defined (PARANOIA)
+/* to be used in one of two possible scenarios */
+static void setup_chroot (char *chroot_dir) {
+  if (geteuid())
+    log_fatal ("you must be root to use chroot");
+
+  if (chroot(chroot_dir)) {
+    log_fatal ("chroot(\"%s\"): %m", chroot_dir);
+  }
+  if (chdir ("/")) {
+    /* probably permission denied */
+    log_fatal ("chdir(\"/\"): %m");
+  }
+}
+#endif /* PARANOIA */
+
 #ifndef UNIT_TEST
 int 
 main(int argc, char **argv) {
@@ -225,6 +254,15 @@ main(int argc, char **argv) {
 	char *traceoutfile = (char *)0;
 #endif
 
+#if defined (PARANOIA)
+	char *set_user   = 0;
+	char *set_group  = 0;
+	char *set_chroot = 0;
+
+	uid_t set_uid = 0;
+	gid_t set_gid = 0;
+#endif /* PARANOIA */
+
         /* Make sure that file descriptors 0 (stdin), 1, (stdout), and
            2 (stderr) are open. To do this, we assume that when we
            open a file the lowest available file descriptor is used. */
@@ -241,14 +279,6 @@ main(int argc, char **argv) {
 	/* Set up the client classification system. */
 	classification_setup ();
 
-	/*
-	 * Set up the signal handlers, currently we only
-	 * have one to ignore sigpipe.
-	 */
-	if (dhcp_handle_signal(SIGPIPE, SIG_IGN) != ISC_R_SUCCESS) {
-		log_fatal("Can't set up signal handler");
-	}
-
 	/* Initialize the omapi system. */
 	result = omapi_init ();
 	if (result != ISC_R_SUCCESS)
@@ -268,7 +298,15 @@ main(int argc, char **argv) {
 		if (!strcmp (argv [i], "-p")) {
 			if (++i == argc)
 				usage ();
-			local_port = validate_port (argv [i]);
+			for (s = argv [i]; *s; s++)
+				if (!isdigit ((unsigned char)*s))
+					log_fatal ("%s: not a valid UDP port",
+					       argv [i]);
+			status = atoi (argv [i]);
+			if (status < 1 || status > 65535)
+				log_fatal ("%s: not a valid UDP port",
+				       argv [i]);
+			local_port = htons (status);
 			log_debug ("binding to user-specified port %d",
 			       ntohs (local_port));
 		} else if (!strcmp (argv [i], "-f")) {
@@ -284,6 +322,20 @@ main(int argc, char **argv) {
 			if (++i == argc)
 				usage ();
 			server = argv [i];
+#if defined (PARANOIA)
+		} else if (!strcmp (argv [i], "-user")) {
+			if (++i == argc)
+				usage ();
+			set_user = argv [i];
+		} else if (!strcmp (argv [i], "-group")) {
+			if (++i == argc)
+				usage ();
+			set_group = argv [i];
+		} else if (!strcmp (argv [i], "-chroot")) {
+			if (++i == argc)
+				usage ();
+			set_chroot = argv [i];
+#endif /* PARANOIA */
 		} else if (!strcmp (argv [i], "-cf")) {
 			if (++i == argc)
 				usage ();
@@ -352,10 +404,6 @@ main(int argc, char **argv) {
 		} else {
 			struct interface_info *tmp =
 				(struct interface_info *)0;
-			if (strlen(argv[i]) >= sizeof(tmp->name))
-				log_fatal("%s: interface name too long "
-					  "(is %ld)",
-					  argv[i], (long)strlen(argv[i]));
 			result = interface_allocate (&tmp, MDL);
 			if (result != ISC_R_SUCCESS)
 				log_fatal ("Insufficient memory to %s %s: %s",
@@ -441,11 +489,49 @@ main(int argc, char **argv) {
 					     trace_seed_stop, MDL);
 #endif
 
+#if defined (PARANOIA)
+	/* get user and group info if those options were given */
+	if (set_user) {
+		struct passwd *tmp_pwd;
+
+		if (geteuid())
+			log_fatal ("you must be root to set user");
+
+		if (!(tmp_pwd = getpwnam(set_user)))
+			log_fatal ("no such user: %s", set_user);
+
+		set_uid = tmp_pwd->pw_uid;
+
+		/* use the user's group as the default gid */
+		if (!set_group)
+			set_gid = tmp_pwd->pw_gid;
+	}
+
+	if (set_group) {
+/* get around the ISC declaration of group */
+#define group real_group
+		struct group *tmp_grp;
+
+		if (geteuid())
+			log_fatal ("you must be root to set group");
+
+		if (!(tmp_grp = getgrnam(set_group)))
+			log_fatal ("no such group: %s", set_group);
+
+		set_gid = tmp_grp->gr_gid;
+#undef group
+	}
+
+#  if defined (EARLY_CHROOT)
+	if (set_chroot) setup_chroot (set_chroot);
+#  endif /* EARLY_CHROOT */
+#endif /* PARANOIA */
+
 	/* Default to the DHCP/BOOTP port. */
 	if (!local_port)
 	{
 		if ((s = getenv ("DHCPD_PORT"))) {
-			local_port = validate_port (s);
+			local_port = htons (atoi (s));
 			log_debug ("binding to environment-specified port %d",
 				   ntohs (local_port));
 		} else {
@@ -503,7 +589,7 @@ main(int argc, char **argv) {
 	}
 
 	/* Get the current time... */
-	time(&cur_time);
+	gettimeofday(&cur_tv, NULL);
 
 	/* Set up the initial dhcp option universe. */
 	initialize_common_option_spaces ();
@@ -568,9 +654,15 @@ main(int argc, char **argv) {
 #endif
 
 #ifdef DHCPv6
-	/* set up DHCPv6 hash */
-	if (!ia_na_new_hash(&ia_active, DEFAULT_HASH_SIZE, MDL)) {
-		log_fatal("Out of memory creating hash for active IA.");
+	/* set up DHCPv6 hashes */
+	if (!ia_new_hash(&ia_na_active, DEFAULT_HASH_SIZE, MDL)) {
+		log_fatal("Out of memory creating hash for active IA_NA.");
+	}
+	if (!ia_new_hash(&ia_ta_active, DEFAULT_HASH_SIZE, MDL)) {
+		log_fatal("Out of memory creating hash for active IA_TA.");
+	}
+	if (!ia_new_hash(&ia_pd_active, DEFAULT_HASH_SIZE, MDL)) {
+		log_fatal("Out of memory creating hash for active IA_PD.");
 	}
 #endif /* DHCPv6 */
 
@@ -579,6 +671,10 @@ main(int argc, char **argv) {
 		log_fatal ("Configuration file errors encountered -- exiting");
 
 	postconf_initialization (quiet);
+ 
+#if defined (PARANOIA) && !defined (EARLY_CHROOT)
+	if (set_chroot) setup_chroot (set_chroot);
+#endif /* PARANOIA && !EARLY_CHROOT */
 
         /* test option should cause an early exit */
  	if (cftest && !lftest) 
@@ -607,6 +703,7 @@ main(int argc, char **argv) {
 	 */
 	if (local_family == AF_INET6) {
 		mark_hosts_unavailable();
+		mark_phosts_unavailable();
 		mark_interfaces_unavailable();
 	}
 #endif /* DHCPv6 */
@@ -656,6 +753,22 @@ main(int argc, char **argv) {
 		else if (pid)
 			exit (0);
 	}
+ 
+#if defined (PARANOIA)
+	/* change uid to the specified one */
+
+	if (set_gid) {
+		if (setgroups (0, (void *)0))
+			log_fatal ("setgroups: %m");
+		if (setgid (set_gid))
+			log_fatal ("setgid(%d): %m", (int) set_gid);
+	}	
+
+	if (set_uid) {
+		if (setuid (set_uid))
+			log_fatal ("setuid(%d): %m", (int) set_uid);
+	}
+#endif /* PARANOIA */
 
 	/* Read previous pid file. */
 	if ((i = open (path_dhcpd_pid, O_RDONLY)) >= 0) {
@@ -678,7 +791,7 @@ main(int argc, char **argv) {
         /* Write new pid file. */
         if ((i = open(path_dhcpd_pid, O_WRONLY|O_CREAT|O_TRUNC, 0644)) >= 0) {
                 sprintf(pbuf, "%d\n", (int) getpid());
-                IGNORE_RET (write(i, pbuf, strlen(pbuf)));
+                write(i, pbuf, strlen(pbuf));
                 close(i);
         } else {
                 log_error("Can't create PID file %s: %m.", path_dhcpd_pid);
@@ -707,7 +820,7 @@ main(int argc, char **argv) {
                 open("/dev/null", O_RDWR);
                 log_perror = 0; /* No sense logging to /dev/null. */
 
-       		IGNORE_RET (chdir("/"));
+                chdir("/");
 	}
 #endif /* !DEBUG */
 
@@ -721,6 +834,8 @@ main(int argc, char **argv) {
 	omapi_set_int_value ((omapi_object_t *)dhcp_control_object,
 			     (omapi_object_t *)0, "state", server_running);
 
+	register_eventhandler(&rw_queue_empty,commit_leases_readerdry);
+	
 	/* Receive packets and dispatch them... */
 	dispatch ();
 
@@ -890,7 +1005,7 @@ void postconf_initialization (int quiet)
 		if (db.len == 4) {
 			memcpy (&limited_broadcast, db.data, 4);
 		} else
-			log_fatal ("invalid broadcast address data length");
+			log_fatal ("invalid remote port data length");
 		data_string_forget (&db, MDL);
 	}
 
@@ -904,7 +1019,7 @@ void postconf_initialization (int quiet)
 		if (db.len == 4) {
 			memcpy (&local_address, db.data, 4);
 		} else
-			log_fatal ("invalid local address data length");
+			log_fatal ("invalid remote port data length");
 		data_string_forget (&db, MDL);
 	}
 
@@ -955,6 +1070,34 @@ void postconf_initialization (int quiet)
 			data_string_forget (&db, MDL);
 		}
 	}
+	
+	oc = lookup_option(&server_universe, options, SV_DELAYED_ACK);
+	if (oc &&
+	    evaluate_option_cache(&db, NULL, NULL, NULL, options, NULL,
+				  &global_scope, oc, MDL)) {
+		if (db.len == 2) {
+			max_outstanding_acks = htons(getUShort(db.data));
+		} else {
+			log_fatal("invalid max delayed ACK count ");
+		}
+		data_string_forget(&db, MDL);
+	}
+
+	oc = lookup_option(&server_universe, options, SV_MAX_ACK_DELAY);
+	if (oc &&
+	    evaluate_option_cache(&db, NULL, NULL, NULL, options, NULL,
+				  &global_scope, oc, MDL)) {
+		u_int32_t timeval;
+
+		if (db.len != 4)
+			log_fatal("invalid max ack delay configuration");
+
+		timeval = getULong(db.data);
+		max_ack_delay_secs  = timeval / 1000000;
+		max_ack_delay_usecs = timeval % 1000000;
+
+		data_string_forget(&db, MDL);
+	}
 
 	/* Don't need the options anymore. */
 	option_state_dereference (&options, MDL);
@@ -1035,6 +1178,10 @@ usage(void) {
 #else /* !DHCPv6 */
 		  "             [-cf config-file] [-lf lease-file]\n"
 #endif /* DHCPv6 */
+#if defined (PARANOIA)
+		   /* meld into the following string */
+		  "             [-user user] [-group group] [-chroot dir]\n"
+#endif /* PARANOIA */
 #if defined (TRACING)
 		  "             [-tf trace-output-file]\n"
 		  "             [-play trace-input-file]\n"
@@ -1210,10 +1357,11 @@ isc_result_t dhcp_io_shutdown (omapi_object_t *obj, void *foo)
 
 static isc_result_t dhcp_io_shutdown_countdown (void *vlp)
 {
-	dhcp_failover_state_t *state;
 #if defined (FAILOVER_PROTOCOL)
+	dhcp_failover_state_t *state;
 	int failover_connection_count = 0;
 #endif
+	struct timeval tv;
 
       oncemore:
 	if (shutdown_state == shutdown_listeners ||
@@ -1299,7 +1447,9 @@ static isc_result_t dhcp_io_shutdown_countdown (void *vlp)
 		shutdown_time = cur_time;
 		goto oncemore;
 	}
-	add_timeout (cur_time + 1,
+	tv.tv_sec = cur_tv.tv_sec + 1;
+	tv.tv_usec = cur_tv.tv_usec;
+	add_timeout (&tv,
 		     (void (*)(void *))dhcp_io_shutdown_countdown, 0, 0, 0);
 	return ISC_R_SUCCESS;
 }
