@@ -1051,6 +1051,7 @@ int supersede_lease (comp, lease, commit, propogate, pimmediate)
 	int pimmediate;
 {
 	struct lease *lp, **lq, *prev;
+	struct timeval tv;
 #if defined (FAILOVER_PROTOCOL)
 	int do_pool_check = 0;
 
@@ -1335,7 +1336,9 @@ int supersede_lease (comp, lease, commit, propogate, pimmediate)
 	    (comp -> sort_time < comp -> pool -> next_event_time ||
 	     comp -> pool -> next_event_time == MIN_TIME)) {
 		comp -> pool -> next_event_time = comp -> sort_time;
-		add_timeout (comp -> pool -> next_event_time,
+		tv . tv_sec = comp -> pool -> next_event_time;
+		tv . tv_usec = 0;
+		add_timeout (&tv,
 			     pool_timer, comp -> pool,
 			     (tvref_t)pool_reference,
 			     (tvunref_t)pool_dereference);
@@ -1512,6 +1515,7 @@ void make_binding_state_transition (struct lease *lease)
 	      case FTS_ABANDONED:
 	      case FTS_RESET:
 		lease -> next_binding_state = FTS_FREE;
+#if defined(FAILOVER_PROTOCOL)
 		/* If we are not in partner_down, leases don't go from
 		   EXPIRED to FREE on a timeout - only on an update.
 		   If we're in partner_down, they expire at mclt past
@@ -1521,6 +1525,7 @@ void make_binding_state_transition (struct lease *lease)
 			lease -> tsfp =
 			    (lease -> pool -> failover_peer -> me.stos +
 			     lease -> pool -> failover_peer -> mclt);
+#endif /* FAILOVER_PROTOCOL */
 		break;
 
 	      case FTS_FREE:
@@ -1753,6 +1758,7 @@ void pool_timer (vpool)
 	struct lease **lptr[RESERVED_LEASES+1];
 	TIME next_expiry = MAX_TIME;
 	int i;
+	struct timeval tv;
 
 	pool = (struct pool *)vpool;
 
@@ -1821,7 +1827,9 @@ void pool_timer (vpool)
 	}
 	if (next_expiry != MAX_TIME) {
 		pool -> next_event_time = next_expiry;
-		add_timeout (pool -> next_event_time, pool_timer, pool,
+		tv . tv_sec = pool -> next_event_time;
+		tv . tv_usec = 0;
+		add_timeout (&tv, pool_timer, pool,
 			     (tvref_t)pool_reference,
 			     (tvunref_t)pool_dereference);
 	} else
@@ -2818,6 +2826,7 @@ void free_everything ()
 
 	cancel_all_timeouts ();
 	relinquish_timeouts ();
+	relinquish_ackqueue();
 	trace_free_all ();
 	group_dereference (&root_group, MDL);
 	executable_statement_dereference (&default_classification_rules, MDL);
